// This code can work as a template for codes calling neBEM toolkit
// The user has to supply
// int main(void)

// In addition, the user has to supply an interface between the code and
// neBEM which contains the following and usually resides in the
// src/Interface directory. Few templates exist there for easy reference.
// int neBEMGetNbPrimitives(void)
// int neBEMGetPrimitive(int prim, int *nvertex,
//                       double xvert[], double yvert[], double zvert[],
//                       double *xnorm, double *ynorm, double *znorm,
//                       int *volref1, int *volref2)
// int neBEMVolumeDescription(int volref1, int *shape1,
//                            int *material1, double *epsilon1,
//                            double *potential1, double *charge1,
//                            int *boundarytype1)
// int neBEMVolumePoint(double x, double y, double z)
// int neBEMGetPeriodicities(int *ix, double *sx, int *iy, double *sy,
//                             int *iz, double *sz)
// The Makefile has to be modified appropriately to incorporate these two
// source codes while building the executable.

// Analyze a micro-fabricated ESP

#include <stdio.h>
#include <assert.h>
#include <time.h>

#include <Interface.h>
#include <Vector.h>
#include <NR.h>
#include <neBEM.h>


int main(void)
{
int SpecifyKnownCharges(void);
int PostProcess(void);	// a typical post-processing function

// beginning of neBEM computation - assign the state variable
neBEMState = 0;
int fstatus = neBEMInitialize();
assert(fstatus == 0);

// The following function uses neBEMGetNbPrimitives, neBEMGetPrimitive,
// neBEMVolumeDescription and neBEMVolumePoint, the last one used sparingly
fstatus = neBEMReadGeometry();
assert(fstatus == 0);

// It is likely that a routine will be supplied either by neBEM or the user
// to analyze the pritimives and to decide the size and shape of elements
// necessary on each primitive
// There is already a routine named AnalyzePrimitiveShape in ReTrim.c.
// A similar routine to determine the primitive characteristics and following
// discretization requirements can be implemented within the neBEMDiscretize
// function.
// This can be made more intelligent by making the larger integer take the
// longer side of the primitive
int **elementNbs;
elementNbs = imatrix(1, NbPrimitives, 1, 2);
for(register int prim = 1; prim <= NbPrimitives; ++prim)
	{
	elementNbs[prim][1] = tmpNbXSegs[prim];
	elementNbs[prim][2] = tmpNbZSegs[prim];
	}
fstatus = neBEMDiscretize(elementNbs);
assert(fstatus == 0);

// Since a boundary condition is likely to be
// associated with each element, this routine can be called only after the
// primitives have been discretized
fstatus = neBEMBoundaryConditions();
assert (fstatus == 0);

NewModel = 1; NewMesh = 1; NewBC = 1; NewPP = 1;
ModelCntr = 3; MeshCntr = 1; BCCntr = 1; PPCntr = 1;
int MaxTimeStep = 1;

// Begin time stepping
for(TimeStep = 1, EndOfTime = 0; TimeStep <= MaxTimeStep; ++TimeStep)
	{
	if(TimeStep > 1) // only the BC changes from one time step to another
		{
		NewModel = 0; NewMesh = 0;
		++BCCntr;
		// CreateDirStr();
		}
	if(TimeStep == MaxTimeStep) EndOfTime = 1;

	// Get the status (location and strength) of known charges / charge densities
	// Pass the function pointer as argument
	fstatus = neBEMKnownCharges(&SpecifyKnownCharges);
	assert(fstatus == 0);
	neBEMState = 8;

	// Finally, request the solution.
	fstatus = neBEMSolve();	// fresh calculation
	assert(fstatus == 0);
	}

// Post-processing can be carried out with separate function(s) written within
// this code, or function(s) residing in a separate source code, or a mix.
fstatus = PostProcess();
assert(fstatus == 0);

// End neBEM gracefully
fstatus = neBEMEnd();
assert(fstatus == 0);

return 0;
}	// main ends


// Assign charge / charge densities on points, lines, surfaces and volumes
// Also assign charge / charge densities on elemnts used to describe the device
// In addition, track these entities and update their locations.
// There is one variable, TimeStep, in neBEM header file that can be handy here
int SpecifyKnownCharges(void)
{
// Assignment of known charges is allowed any time after the number of elemnts
// has been decided, the element array has been allocated and the discretization
// completed.
if(neBEMState <= 4)
  {
  printf("Charge can be specfied only beyond state 4 ...\n");
  printf("returning ...\n");
  return(-1);
  }

// Assign linear or surface charge densities on the mesh
// This can also be carried out by specifying a dielectric interface to be
// of type 5, and assigning a charge for the complete primitive at one go.
// Please check the neBEMInterface.c source code.
for(register unsigned int ele = 1; ele <= NbElements; ++ele)
  {
  (EleArr+ele-1)->Assigned = 0.0;
  }

// Points with known charge
NbPtsKnCh = 0;	// zero only!
if(NbPtsKnCh)
  {
	// allocation necessary only for the first time, or made elsewhere?
	// or, reallocate as the numnber of charged particles increase in number?
	// define the geometrical parameters of the device
	double rCathode = 45.0e-3;
	double lCathode = 17.0e-3;
	// double rAnode = 0.8e-3;
	// double lAnode = 0.5*lCathode;
	// double rLangmuir = 0.25e-3;
	// double lLangmuir = 0.05*lCathode;
	const double ST_PI = 3.14159265;
	const double q_e = -1.60217606e-19;

	double rad, theta, xpos, ypos, zpos, charge;

  if(TimeStep == 1)PtKnChArr = (PtKnCh *) malloc(NbPtsKnCh * sizeof(PtKnCh));

  for(register unsigned int point = 1; point <= NbPtsKnCh; ++point)
    {
		srand(time(0));

		double norm_rand = (double)rand() / (double)RAND_MAX;
		rad = rCathode * norm_rand;
		norm_rand = (double)rand() / (double)RAND_MAX;	// decoupled from rad
		theta = 2.0 * ST_PI * norm_rand;
		xpos = rad * cos(theta);
		ypos = rad * sin(theta);
		norm_rand = (double)rand() / (double)RAND_MAX;
		zpos = lCathode * norm_rand;

		// How to make sure that these locations are not inside one of the volumes
		// such as the anode wire, Langmuir probe?

		// singly / multiply charged
		charge = 10000.0*q_e;	// equivalent to ten thousand electrons
		norm_rand = (double)rand() / (double)RAND_MAX;
		if(norm_rand < 0.5) charge *= -1.0;	// switch ionic polalrity

		// remove the points that are outside the vacuum vessel
    (PtKnChArr+point-1)->Nb = point;
    (PtKnChArr+point-1)->P.X = xpos;
    (PtKnChArr+point-1)->P.Y = ypos;
    (PtKnChArr+point-1)->P.Z = zpos;
    (PtKnChArr+point-1)->Assigned = charge; // this is not density
    }
  }

// The following needs to be dealt in a manner similar to the above
// Lines with known charge density
NbLinesKnCh = 0;
// Areas with known charge density
NbAreasKnCh = 0;
// Volumes with known charge density
NbVolsKnCh = 0;

return(0);
}	// end of UpdateKnownCharges
