// An example that creates a TPC field cage
// Several improvements are necessary:
// (1) Rectangular anode and cathode should be optional. It should be possible
// to create circular electrodes as well.
// (2) It should be possible to segment strips in the vertical (Z) direction,
// since although there is symmetry in the azimuthal direction leading to
// same charge density distribution in that direction, the charge density is
// expected to vary substantially in the Z direction. In fact, it is known that
// there will be higher charge densities at the strip edges.
// (3) The distance, number of strips, voltage jump calculations should be
// checked using a simple geometry with only a few strips.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>

#define STPI 3.14159265

int main(void)
{
// User inputs - can be read in from an input file or hash-defined, if desired.
// Or, even better, for a truly elaborate device, use a separate input file.
// Another option could be to have ".def" file which is #included at the
// beginning of a device code, as has been done in the micromegas device codes.
// The convenience of using separate def or input files is to avoid scrolling
// too often. There can, of course, be many such files possibly representing
// each component of a device.
char DeviceInDirName[256], DeviceInFileName[256], ModelName[256], DeviceOutDirName[256];
char *homedir;
homedir = getenv("HOME");
assert(homedir != NULL);
printf("# user homedir is %s\n", homedir);
strcpy(DeviceInDirName, homedir);
strcat(DeviceInDirName, "/neBEMWD/Inputs/FieldCage/");  // existing
strcpy(ModelName, "Model1");  // do NOT put an appending /
strcpy(DeviceOutDirName, homedir);
strcat(DeviceOutDirName, "/neBEMWD/Outputs/FieldCage/");
strcat(DeviceOutDirName, ModelName);
strcat(DeviceOutDirName, "/");

// preparation of necessary input files
char DeviceInDir[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir, DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// The surfaces that define the device are interfaces between these volumes
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;
{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 11, 4, 1.0, 0.0, 0.0);

// Wall Dielectric: VolRef4
// ++NbVolumes;
// CreateVolume(ModelInDir, NbVolumes, 0, 12, 4, 5.0, 0.0, 0.0);

// Teflon dielectric: VolRef5
// ++NbVolumes;
// CreateVolume(ModelInDir, NbVolumes, 4, 13, 4, 1.005, 0.0, 0.0);
}	// Creation of volume reference data files over.

// Geometrical parameters of the cylinder
// Refer to Fig.1 in fcReport_1.doc and related sketches
// The wall is mentioned to be beginning at X=R (160cm)
// Thickness of the copper strips is designated by T (not defined!)
// For presenting the results, Delta-s have been used that denote distances
// from the strip side, rather than from the axis of the field cage.
// T and several other parameters have been defined in figures 1a and 1b
// in fReport_2.doc in relation to the 3 TPC field cage structures that
// have been considered (DESY prototype, MPI-Munich LC TPC prototype and
// VICTORIA TPC
// At present, we are working only on the first prototype.
int NbSegments = 16;
double tStripT = 50.0e-6;		// written mcm in fcReport_2.doc (micro-meter?)
double tWallW = 8.7e-3;			// wall thickness is 30mm
double rCageInner = 125.0e-3;
double rAnode = rCageInner + tWallW;
double rCathode = rAnode;
double vAnode = 0.0;				// grounded
double vCathode = -100.0e3;	// -100 kV - expected field strength: -400V/cm
double vGroundScreen = 0.0;	// -100 kV - expected field strength: -400V/cm
double epsWall = 2.0;					// relative dielectric coefficient of the wall
double dWallGroundScreenD = 0.0;	// right on the wall exterior?
double lGEMH = 700.0e-3;		// distance upto GEM foil
double lFieldCageL = lGEMH;
double lStripL = 1.6e-3;
double lStripPitchS = 2.8e-3;
double dFromAnodeA = 0.5*lStripPitchS;
double dFromCathodeB = 0.5*lStripPitchS;
double dFromAnode = dFromAnodeA - 0.5*lStripL;	// dist from edge of strip
double dFromCathode = dFromCathodeB - 0.5*lStripL;	// dist from edge of strip

/*
lGEMH = 10.0;
lFieldCageL = lGEMH;
lStripL = 1.0;
lStripPitchS = 2.0;
dFromAnodeA = 0.5*lStripPitchS;
dFromCathodeB = 0.5*lStripPitchS;
dFromAnode = dFromAnodeA - 0.5*lStripL;	// dist from edge of strip
dFromCathode = dFromCathodeB - 0.5*lStripL;	// dist from edge of strip
*/

printf("\n# Field Cage parameters ===>\n");
printf("# NbSegments: %d\n", NbSegments);
printf("# tStripT: %lg, tWallW: %lg\n", tStripT, tWallW);
printf("# rCageInner: %lg, rAnode: %lg, rCathode: %lg\n",
				rCageInner, rAnode, rCathode);
printf("# vAnode: %lg, vCathode: %lg, vGroundScreen: %lg, epsWall: %lg\n",
				vAnode, vCathode, vGroundScreen, epsWall);
printf("# dWallGroundScreenD: %lg, lGEMH: %lg\n", dWallGroundScreenD, lGEMH);
printf("# lStripL: %lg, lStripPitchS: %lg, lFieldCageL: %lg\n",
				lStripL, lStripPitchS, lFieldCageL);
printf("# dFromAnodeA: %lg, dFromCathodeB: %lg\n", dFromAnodeA, dFromCathodeB);
printf("# dFromAnode: %lg, dFromCathode: %lg\n", dFromAnode, dFromCathode);

// Start building the device using primitives 
int NbPrimitives = 0;

// Component 1 - Anode disc
{
// latitudes
// how about considering a square plate to begin with?
int nvertex= 4, NbXSegs = 11, NbYSegs = 11;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

// Anode disc made of rectangles
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, vAnode, 0.0);

double VolRef1 = 1; double VolRef2 = NbVolumes;

++NbPrimitives;

// compute vertices
xvert[0] = -rAnode;
yvert[0] = -rAnode;
zvert[0] = 0.0;
xvert[1] = rAnode;
yvert[1] = -rAnode;
zvert[1] = 0.0;
xvert[2] = rAnode;
yvert[2] = rAnode;
zvert[2] = 0.0;
xvert[3] = -rAnode;
yvert[3] = rAnode;
zvert[3] = 0.0;
xnorm = 0.0;
ynorm = 0.0;
znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
           		PeriodicTypeX, PeriodicInX, XPeriod,
           		PeriodicTypeY, PeriodicInY, YPeriod,
           		PeriodicTypeZ, PeriodicInZ, ZPeriod);
}

// Component 2 - Cathode disc
{
// latitudes
// how about considering a square plate to begin with?
int nvertex= 4, NbXSegs = 11, NbYSegs = 11;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

// Cathode disc made of rectangles
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, vCathode, 0.0);

double VolRef1 = 1; double VolRef2 = NbVolumes;

++NbPrimitives;

// compute vertices
xvert[0] = -rCathode;
yvert[0] = -rCathode;
zvert[0] = lFieldCageL;
xvert[1] = rCathode;
yvert[1] = -rCathode;
zvert[1] = lFieldCageL;
xvert[2] = rCathode;
yvert[2] = rCathode;
zvert[2] = lFieldCageL;
xvert[3] = -rCathode;
yvert[3] = rCathode;
zvert[3] = lFieldCageL;
xnorm = 0.0;
ynorm = 0.0;
znorm = 1.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
           		PeriodicTypeX, PeriodicInX, XPeriod,
           		PeriodicTypeY, PeriodicInY, YPeriod,
           		PeriodicTypeZ, PeriodicInZ, ZPeriod);
}

// Component 3 - Field Cage
{

// field cage made of rectangular plates
{
int NbOfStrips;
int nvertex= 4, NbXSegs = 1, NbYSegs = 1;

{	// basic parameters for geoemtry and potential
double StripSpan = lFieldCageL - dFromAnode - dFromCathode;
if(StripSpan < lStripL)
	{
	printf("# Not enough length to place a strip ... exiting ...\n");
	return -1;
	}

NbOfStrips = (int)(StripSpan / (lStripPitchS+lStripL));
++NbOfStrips;

if((NbOfStrips > 1) && (lStripL > lStripPitchS))
	{	// check whether strip pitch is greater than strip length
	printf("# Strips will overlap ... exiting ...\n");
	return -1;
	}

printf("\n# StripSpan: %lg, NbOfStrips: %d\n", StripSpan, NbOfStrips);
}	// basic parameters for geometry and potential ends

printf("# zLow\tzHigh\tzMid\tzDiff\tvStrip\n");
printf("%lg\t%lg\t%lg\t%lg\t%lg\n", 0.0, 0.0, 0.0, 0.0, vAnode);
double zMidOld = 0.0;
for(register int strip = 1; strip <= NbOfStrips; ++strip)
	{
	// Starting point from the anode plane may differ from lStripPitchS
	double zLow = dFromAnode
								+ ((double)strip - 1.0) * (lStripPitchS + lStripL);
	double zHigh = zLow + lStripL;
	double zMid = 0.5 * (zLow + zHigh);

	// Field cage side wall made of strips: different VolRefs for each strip
	// since each strip is at a different voltage
	// Each strip is segmented as rectangles that are at the same potential
	double vStrip = vAnode + zMid * (vCathode - vAnode) / lFieldCageL;

	++NbVolumes;
	CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, vStrip, 0.0);

	double VolRef1 = 1; double VolRef2 = NbVolumes;

	printf("%lg\t%lg\t%lg\t%lg\t%lg\n", zLow, zHigh, zMid, zMid-zMidOld, vStrip);

	double dTheta = 2.0*STPI/NbSegments;
	double avTheta, bgTheta, enTheta;
	for(register int segment=1; segment<=NbSegments; ++segment)
		{
		double xvert[4], yvert[4], zvert[4];
		double xnorm, ynorm, znorm;
		int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
		int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
		double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

		++NbPrimitives;

		avTheta = (segment-1) * dTheta;
		bgTheta = avTheta - 0.5*dTheta;
		enTheta = avTheta + 0.5*dTheta;

		// compute vertices
		xvert[0] = rCageInner * cos(bgTheta);
		yvert[0] = rCageInner * sin(bgTheta);
		zvert[0] = zLow;
		xvert[1] = xvert[0];
		yvert[1] = yvert[0];
		zvert[1] = zHigh;
		xvert[2] = rCageInner * cos(enTheta);
		yvert[2] = rCageInner * sin(enTheta);
		zvert[2] = zHigh;
		xvert[3] = xvert[2];
		yvert[3] = yvert[2];
		zvert[3] = zLow;
		xnorm = cos(avTheta);
		ynorm = sin(avTheta);
		znorm = 0.0;

		CreatePolygon(fModelInp, fData, ModelInDir,
									NbPrimitives,
									nvertex, xvert, yvert, zvert,
									xnorm, ynorm, znorm,
									VolRef1, VolRef2, NbXSegs, NbYSegs,
  	           		PeriodicTypeX, PeriodicInX, XPeriod,
   	           		PeriodicTypeY, PeriodicInY, YPeriod,
   	           		PeriodicTypeZ, PeriodicInZ, ZPeriod);
		}	// for segments

	zMidOld = zMid;
	}	// for strips
printf("%lg\t%lg\t%lg\t%lg\t%lg\n",
				lFieldCageL, lFieldCageL, lFieldCageL, lFieldCageL-zMidOld, vCathode);
}	// field cage made of rectangular plates, ends

} // Component 3 - field cage


// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 1;
if(NbMaps)
	{
	NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
	NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
	MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
	MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
	MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

	// Map 1
	NbXMap[1] = 1; NbYMap[1] = 51; NbZMap[1] = 51; // points along each axis
	MapLX[1] = 0.0;
	MapLY[1] = 3.0 * rCathode;
	MapLZ[1] = 1.5 * lFieldCageL;	// lengths of the map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0; MapZMid[1] = lFieldCageL/2.0;	// mid-points
	NbBlock[1] = NbZMap[1];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 1;
if(NbLines)
	{
	NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
	YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
	XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
	ZStop = dvector(1, NbLines);

	// Line 1
	XStart[1] = 0.0; YStart[1] = 0.0; ZStart[1] = -0.25*lFieldCageL;
	XStop[1] = 0.0; YStop[1] = 0.0; ZStop[1] = 1.25*lFieldCageL;
	NbSegments[1] = 1000;
	}

// Points - can be multiple points
NbPoints = 0;
if(NbPoints)
  {
  XPoint = dvector(1, (long)NbPoints); YPoint = dvector(1, (long)NbPoints);
  ZPoint = dvector(1, (long)NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
