// An example that creates a glow discharge plasma device

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>

#define STPI 3.14159265

int main(void)
{
// User inputs - can be read in from an input file or hash-defined, if desired.
// Or, even better, for a truly elaborate device, use a separate input file.
// Another option could be to have ".def" file which is #included at the
// beginning of a device code, as has been done in the micromegas device codes.
// The convenience of using separate def or input files is to avoid scrolling
// too often. There can, of course, be many such files possibly representing
// each component of a device.
char DeviceInDirName[256], DeviceInFileName[256], ModelName[256], DeviceOutDirName[256];
char *homedir;
homedir = getenv("HOME");
assert(homedir != NULL);
printf("user homedir is %s\n", homedir);
strcpy(DeviceInDirName, homedir);
strcat(DeviceInDirName, "/neBEMWD/Inputs/GdpDevice/");  // existing
strcpy(ModelName, "Model1");  // do NOT put an appending /
strcpy(DeviceOutDirName, homedir);
strcat(DeviceOutDirName, "/neBEMWD/Outputs/GdpDevice/");
strcat(DeviceOutDirName, ModelName);
strcat(DeviceOutDirName, "/");

// preparation of necessary input files
char DeviceInDir[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir, DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// For this analysis, there are six volumes
// The rectangular surface is at the interface of two volumes (1 and 2)
// The triangular surface is at the interface of two volumes (3 and 4)
// The wire surface is at the interface of two volumes (5 and 6)
// Vacuum (VolRef: 1), Conductor 1 (VolRef: 2)
// Dielectric 1 (VolRef: 3), Dielectric 2 (VolRef: 4)
// Gas (VolRef: 5), Conductor 2 (VolRef: 6)
// The surfaces that define the device are interfaces between these volumes
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;
{
// Cathode conductor: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 0.0, 0.0);

// Anode conductor: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 2, 1, 1, 0.0, 1000.0, 0.0);

// Langmuir probe conductor: VolRef3
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 2, 1, 3, 0.0, 0.0, 0.0);

// Argon Dielectric: VolRef4
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 11, 4, 5.0, 0.0, 0.0);

// Teflon dielectric: VolRef5
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 12, 4, 1.005, 0.0, 0.0);
}	// Creation of volume reference data files over.

// Geometrical parameters of the cylinder
int NbSegments = 16;
double rCathode = 45.0e-3;
double lCathode = 17.0e-2;
double rAnode = 0.8e-3;
double lAnode = lCathode;
double rLangmuir = 0.25e-3;
double lLangmuir = lCathode;

// Start building the device using primitives 
int NbPrimitives = 0;

// Component 1 - Cathode cylinder
{

// cathode cylinder made of rectangular plates
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

nvertex = 4; VolRef1 = 1; VolRef2 = 4; NbXSegs = 11; NbYSegs = 11;

double dTheta = 2.0*STPI/NbSegments;
double avTheta, bgTheta, enTheta;
for(register int segment=1; segment<=NbSegments; ++segment)
	{
	++NbPrimitives;

	avTheta = (segment-1) * dTheta;
	bgTheta = avTheta - 0.5*dTheta;
	enTheta = avTheta + 0.5*dTheta;

	// compute vertices
	xvert[0] = rCathode * cos(bgTheta);
	yvert[0] = rCathode * sin(bgTheta);
	zvert[0] = 0.0;
	xvert[1] = xvert[0];
	yvert[1] = yvert[0];
	zvert[1] = lCathode;
	xvert[2] = rCathode * cos(enTheta);
	yvert[2] = rCathode * sin(enTheta);
	zvert[2] = lCathode;
	xvert[3] = xvert[2];
	yvert[3] = yvert[2];
	zvert[3] = 0.0;
	xnorm = cos(avTheta);
	ynorm = sin(avTheta);
	znorm = 0.0;

	CreatePolygon(fModelInp, fData, ModelInDir,
								NbPrimitives,
								nvertex, xvert, yvert, zvert,
								xnorm, ynorm, znorm,
								VolRef1, VolRef2, NbXSegs, NbYSegs,
  	           PeriodicTypeX, PeriodicInX, XPeriod,
   	           PeriodicTypeY, PeriodicInY, YPeriod,
   	           PeriodicTypeZ, PeriodicInZ, ZPeriod);
	}	// for segments
}	// RectangularPlate ends

} // Component 1 cathode cylinder ends


// Component 2 - Anode wire
{

// a thin wire
{
int nvertex;
int VolRef1, VolRef2, NbSegs;
double radius;
double xvert[2], yvert[2], zvert[2];
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 2; NbSegs = 11; VolRef1 = 2; VolRef2 = 4;
radius = rAnode;

// compute vertices
xvert[0] = 0.0; yvert[0] = 0.0; zvert[0] = 0.0;
xvert[1] = 0.0; yvert[1] = 0.0; zvert[1] = lAnode;

CreateLine(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							radius, VolRef1, VolRef2, NbSegs,
  	           PeriodicTypeX, PeriodicInX, XPeriod,
   	           PeriodicTypeY, PeriodicInY, YPeriod,
   	           PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// wire ends

} // Component 2 anode wire ends


// Component 3 - Langmuir probe wire
{

// a thin wire
{
int nvertex;
int VolRef1, VolRef2, NbSegs;
double radius;
double xvert[2], yvert[2], zvert[2];
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 2; NbSegs = 11; VolRef1 = 3; VolRef2 = 4;
radius = rLangmuir;

// compute vertices
xvert[0] = 0.0; yvert[0] = 0.75*rCathode; zvert[0] = 0.0;
xvert[1] = 0.0; yvert[1] = 0.75*rCathode; zvert[1] = lLangmuir;

CreateLine(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							radius, VolRef1, VolRef2, NbSegs,
  	           PeriodicTypeX, PeriodicInX, XPeriod,
   	           PeriodicTypeY, PeriodicInY, YPeriod,
   	           PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// wire ends

} // Component 3 Langmuir probe wire ends


// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 1;
if(NbMaps)
	{
	NbXMap = ivector(1, (long)NbMaps); NbYMap = ivector(1, (long)NbMaps);
	NbZMap = ivector(1, (long)NbMaps); MapLX = dvector(1, (long)NbMaps);
	MapLY = dvector(1, (long)NbMaps); MapLZ = dvector(1, (long)NbMaps);
	MapXMid = dvector(1, (long)NbMaps); MapYMid = dvector(1, (long)NbMaps);
	MapZMid = dvector(1, (long)NbMaps); NbBlock = ivector(1, (long)NbMaps);

	// Map 1
	NbXMap[1] = 51; NbYMap[1] = 51; NbZMap[1] = 1; // points along each axis
	MapLX[1] = 2.0*rCathode;
	MapLY[1] = 2.0*rCathode;
	MapLZ[1] = 0.0;	// lengths of the map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0; MapZMid[1] = lCathode/2.0;	// mid-points
	NbBlock[1] = NbYMap[1];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 1;
if(NbLines)
	{
	NbSegments = ivector(1, (long)NbLines); XStart = dvector(1, (long)NbLines);
	YStart = dvector(1, (long)NbLines); ZStart = dvector(1, (long)NbLines);
	XStop = dvector(1, (long)NbLines); YStop = dvector(1, (long)NbLines);
	ZStop = dvector(1, (long)NbLines);

	// Line 1
	XStart[1] = -rCathode; YStart[1] = 0.0; ZStart[1] = 0.5*lCathode;
	XStop[1] = rCathode; YStop[1] = 0.0; ZStop[1] = 0.5*lCathode;
	NbSegments[1] = 100;
	}

// Points - can be multiple points
NbPoints = 0;
if(NbPoints)
  {
  XPoint = dvector(1, (long)NbPoints); YPoint = dvector(1, (long)NbPoints);
  ZPoint = dvector(1, (long)NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
