// Any device is composed of primitives of two kinds:
// wire (1D) and flat surface (2D).
// The flat surfaces can be either right triangular and rectangular.
// In the case of a right triangular primitive, a sequence is maintained such
// that the second vertex is the right corner.
// By definition, primitive of the first kind can be used to represent only
// few very special geometries. On the other hand, the second kind can
// represent 3D geometries of any kind.

// While defining the device, it is also important to define the volumes that
// define any given primitive. This is so because, the boundary conditions
// applicable to an interface depends on the properties of the volumes on its
// either side. So, each primitive has two volumes associated to it. As a
// result, it is important to identify the volumes that we need to supply
// in order to completely sepcify the problem. In addition, please note
// that primitives should be defined such that they are at the interface
// of two volumes only.
//
// The input files are defined in the following manner:
// A device input directory contains all the information for a device under
// study, one subdirectory for each model representation; the model
// subdirectories in their turn contain files that store data on each of the
// primitives, the volume files and the map file.
// The device directory is assumed to be already existing.
//
// An example that creates an Iarocci tube composed of an outer cylinder
// and a wire at the middle (drift tube / straw tube)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <assert.h>

#include <NR.h>
#include <Vector.h>
#include <Device.h>


int main(void)
{
// User inputs - can be read in from an input file or hash-defined, if desired.
// Or, even better, for a truly elaborate device, use a separate input file.
// Another option could be to have ".def" file which is #included at the
// beginning of a device code, as has been done in the micromegas device codes.
char DeviceInDirName[256], DeviceInFileName[256], ModelName[256], DeviceOutDirName[256];
char *homedir;
homedir = getenv("HOME");
assert(homedir != NULL);
printf("user homedir is %s\n", homedir);
strcpy(DeviceInDirName, homedir);
strcat(DeviceInDirName, "/neBEMWD/Inputs/DetSim/Iarocci/");  // existing
strcpy(ModelName, "Model1");  // do NOT put an appending /
strcpy(DeviceOutDirName, homedir);
strcat(DeviceOutDirName, "/neBEMWD/Outputs/DetSim/Iarocci/");
strcat(DeviceOutDirName, ModelName);
strcat(DeviceOutDirName, "/");

// preparation of necessary input files
char DeviceInDir[256], ModelSubDir[256], ModelInDir[256];
FILE *fModelInp;
strcpy(DeviceInDir, DeviceInDirName);	// existing directory
strcpy(ModelSubDir, ModelName);	// to be created
fModelInp = PrepareInputFiles(DeviceInDir, ModelSubDir,
															DeviceInFileName, ModelInDir);

// file-handler for device data file used for viewing
FILE *fData;
fData = PrepareViewingFiles(ModelInDir);

// Create volume reference data
// For this analysis, there are six volumes
// The rectangular surface is at the interface of two volumes (1 and 2)
// The triangular surface is at the interface of two volumes (3 and 4)
// The wire surface is at the interface of two volumes (5 and 6)
// Vacuum (VolRef: 1), Conductor 1 (VolRef: 2)
// Dielectric 1 (VolRef: 3), Dielectric 2 (VolRef: 4)
// Gas (VolRef: 5), Conductor 2 (VolRef: 6)
// The surfaces that define the device are interfaces between these volumes
// void CreateVolume(char ModelInDir[], int VolRef,
//                   int VolShape, int VolMaterial, int VolBoundaryType,
//                   double VolEpsilon, double VolPotential, double VolCharge);
int NbVolumes = 0;

{
// Gas: VolRef1
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 0, 0, 4, 1.0, 0.0, 0.0);

// Conductor surface: VolRef2
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 4, 1, 1, 0.0, 0.0, 0.0);

// Conductor (wire): VolRef3
++NbVolumes;
CreateVolume(ModelInDir, NbVolumes, 2, 2, 1, 0.0, 1000.0, 0.0);
}	// Creation of volume reference data files over.


// Start building the device using primitives 
int NbPrimitives = 0;

// Component 1 - the rectangular tube composed of four rectangular plates
{

// Rectangular plate on Xp
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;

// compute vertices
xvert[0] = +2.50e-3; yvert[0] = -2.50e-3; zvert[0] = -25.00e-3;
xvert[1] = +2.50e-3; yvert[1] = -2.50e-3; zvert[1] = +25.00e-3;
xvert[2] = +2.50e-3; yvert[2] = +2.50e-3; zvert[2] = +25.00e-3;
xvert[3] = +2.50e-3; yvert[3] = +2.50e-3; zvert[3] = -25.00e-3;
xnorm = 1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// RectangularPlate on Xp

// Rectangular plate on Xm
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;

// compute vertices
xvert[0] = -2.50e-3; yvert[0] = -2.50e-3; zvert[0] = -25.00e-3;
xvert[1] = -2.50e-3; yvert[1] = -2.50e-3; zvert[1] = +25.00e-3;
xvert[2] = -2.50e-3; yvert[2] = +2.50e-3; zvert[2] = +25.00e-3;
xvert[3] = -2.50e-3; yvert[3] = +2.50e-3; zvert[3] = -25.00e-3;
xnorm = -1.0; ynorm = 0.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// RectangularPlate on Xm

// Rectangular plate on Yp
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;

// compute vertices
xvert[0] = -2.50e-3; yvert[0] = +2.50e-3; zvert[0] = -25.00e-3;
xvert[1] = -2.50e-3; yvert[1] = +2.50e-3; zvert[1] = +25.00e-3;
xvert[2] = +2.50e-3; yvert[2] = +2.50e-3; zvert[2] = +25.00e-3;
xvert[3] = +2.50e-3; yvert[3] = +2.50e-3; zvert[3] = -25.00e-3;
xnorm = 0.0; ynorm = 1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// RectangularPlate on Yp

// Rectangular plate on Ym
{
int nvertex, VolRef1, VolRef2, NbXSegs, NbYSegs;
double xvert[4], yvert[4], zvert[4];
double xnorm, ynorm, znorm;
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

nvertex = 4; VolRef1 = 2; VolRef2 = 1; NbXSegs = 11; NbYSegs = 11;

// compute vertices
xvert[0] = -2.50e-3; yvert[0] = -2.50e-3; zvert[0] = -25.00e-3;
xvert[1] = -2.50e-3; yvert[1] = -2.50e-3; zvert[1] = +25.00e-3;
xvert[2] = +2.50e-3; yvert[2] = -2.50e-3; zvert[2] = +25.00e-3;
xvert[3] = +2.50e-3; yvert[3] = -2.50e-3; zvert[3] = -25.00e-3;
xnorm = 0.0; ynorm = -1.0; znorm = 0.0;

CreatePolygon(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							xnorm, ynorm, znorm,
							VolRef1, VolRef2, NbXSegs, NbYSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// RectangularPlate on Ym

} // Rectangular tube ends


// Component 2 - a thin wire at the center of the tube
{

// a thin wire
{
int nvertex;
int VolRef1, VolRef2, NbSegs;
double radius = 25.0e-6;	// radius
double xvert[2], yvert[2], zvert[2];
int PeriodicTypeX = 0, PeriodicTypeY = 0, PeriodicTypeZ = 0;
int PeriodicInX = 0, PeriodicInY = 0, PeriodicInZ = 0;
double XPeriod = 0.0, YPeriod = 0.0, ZPeriod = 0.0;

++NbPrimitives;

// dielectric surrounding the wire - can give rise
// to non-physical scenario: there is no interface between vacuum
// and these dielctric materials - this should be vacuum in order
// to avoid a new dielectric-dielectric interface
nvertex = 2; NbSegs = 11; VolRef1 = 3; VolRef2 = 1;

// compute vertices
xvert[0] = 0.0; yvert[0] = 0.0; zvert[0] = -25.0e-3;
xvert[1] = 0.0; yvert[1] = 0.0; zvert[1] = 25.0e-3;

CreateLine(fModelInp, fData, ModelInDir,
							NbPrimitives,
							nvertex, xvert, yvert, zvert,
							radius, VolRef1, VolRef2, NbSegs,
              PeriodicTypeX, PeriodicInX, XPeriod,
              PeriodicTypeY, PeriodicInY, YPeriod,
              PeriodicTypeZ, PeriodicInZ, ZPeriod);
}	// wire ends

} // Component 3 ends

// Close device data file
fclose(fData);


// Creation of map file to evaluate properties on maps (2d grids), along lines
// and at given points
{
// data that goes into the Map files
int NbMaps, NbLines, NbPoints;
int *NbXMap, *NbYMap, *NbZMap;
double *MapLX, *MapLY, *MapLZ;
double *MapXMid, *MapYMid, *MapZMid;
int *NbBlock;
int *NbSegments;
double *XStart, *YStart, *ZStart, *XStop, *YStop, *ZStop;
double *XPoint, *YPoint, *ZPoint;

// Maps - there can be multiple maps
NbMaps = 1;
if(NbMaps)
	{
	NbXMap = ivector(1, NbMaps); NbYMap = ivector(1, NbMaps);
	NbZMap = ivector(1, NbMaps); MapLX = dvector(1, NbMaps);
	MapLY = dvector(1, NbMaps); MapLZ = dvector(1, NbMaps);
	MapXMid = dvector(1, NbMaps); MapYMid = dvector(1, NbMaps);
	MapZMid = dvector(1, NbMaps); NbBlock = ivector(1, NbMaps);

	// Map 1
	NbXMap[1] = 50; NbYMap[1] = 50; NbZMap[1] = 1; // points along each axis
	MapLX[1] = 2.500e-3; MapLY[1] = 2.500e-3; MapLZ[1] = 0.0;	// lengths of map
	MapXMid[1] = 0.0; MapYMid[1] = 0.0; MapZMid[1] = 0.0;	// mid-points
	NbBlock[1] = NbYMap[1];	// length of each block (in relation to gnuplot)
	}

// Lines - can be multiple lines
NbLines = 1;
if(NbLines)
	{
	NbSegments = ivector(1, NbLines); XStart = dvector(1, NbLines);
	YStart = dvector(1, NbLines); ZStart = dvector(1, NbLines);
	XStop = dvector(1, NbLines); YStop = dvector(1, NbLines);
	ZStop = dvector(1, NbLines);

	// Line 1
	XStart[1] = 0.0; YStart[1] = -2.50e-3; ZStart[1] = 0.0;
	XStop[1] = 0.0; YStop[1] = +2.50e-3; ZStop[1] = 0.0;
	NbSegments[1] = 100;
	}

// Points - can be multiple point
NbPoints = 0;
if(NbPoints)
  {
  XPoint = dvector(1, NbPoints); YPoint = dvector(1, NbPoints);
  ZPoint = dvector(1, NbPoints);

  for(register int point = 1; point <= NbPoints; ++point)
    {
    XPoint[point] = 0.0; YPoint[point] = 0.0; ZPoint[point] = 0.0;
    }
  }

CreateMapFile(ModelInDir,
               NbMaps, NbBlock,
               NbXMap, NbYMap, NbZMap,
               MapLX, MapLY, MapLZ,
               MapXMid, MapYMid, MapZMid,
               NbLines, NbSegments,
               XStart, YStart, ZStart, XStop, YStop, ZStop,
               NbPoints, XPoint, YPoint, ZPoint);
}	// Map creation ends


// Complete the DeviceInputFile
int NbBCondns = 1;
EndDeviceInputFile(fModelInp, NbBCondns,
										DeviceOutDirName, ModelInDir, NbPrimitives, NbVolumes);

return 0;
}	// main ends
